%include "win32.inc"
%include "macros.inc"

%include "kernel32.inc"

section .data

; --- screen messages ---

string greeting, "Microplanet Gravity X-Face patch v1.0", CRLF, "Gravity versions supported: 2.3, 2.5", CRLF, CRLF, "This program will: ", CRLF, " - Check if the Gravity.exe file matches a known version", CRLF, " - Make a backup of Gravity.exe to Gravity.exe.bak", CRLF, " - Patch Gravity.exe to add the X-Face display feature", CRLF, CRLF, "The patched Gravity.exe will require WinFace.dll.", CRLF, CRLF
string question, "Do you want to proceed with this? (enter y if you do) "
string notice1, CRLF, CRLF, "Verifying Gravity.exe...", CRLF
string notice2, "  File opened", CRLF
notice3	db "  File size OK, Gravity version 2."
 notice3versionminor	db "x build "
 notice3buildx100	db "xx00", CRLF
notice3_end:
%define notice3_size notice3_end-notice3
string notice4, "  Integrity check passed", CRLF, "Copying Gravity.exe to Gravity.exe.bak...", CRLF
string notice5, "  Backup created", CRLF, "Patching Gravity.exe...", CRLF
string notice6, "  Patch successfully applied", CRLF
string error, "Error: "
string wrong_filesize, "File size does not match any known version"
string integrity_fail, "File integrity failure; patch already applied?"
string error_gravity_damaged, "File writing error, Gravity.exe may be damaged - restore backup", CRLF
string done, CRLF, "Press any key to exit..."

; --- filenames ---

string filename, "Gravity.exe", 0
string backup_filename, "Gravity.exe.bak", 0

; --- some buffers ---

dummy		dd 0

string charbuf, " "
err_msg_ptr	dd 0

buf		times 8 dd 0

; --- patch data ---

%define GRAVITY_FILE_SIZE_2_5	2506752
%define GRAVITY_FILE_SIZE_2_3	2523136

; note: file offsets and example code are commented for version 2.50 only

; part 1 of the patch; the code which will jump to part 2. this will be placed
; at offset 07641h in the file, which corresponds to 04007641h at runtime.
; Gravity reaches that code when it has header lines to process. At [ebp+17]
; there will be a pointer to the headers.

; original code:

; 0040763F  84C0              test    al, al
; 00407641  0F85 E0020000     jnz     00407927
; 00407647  8B5D 11           mov     ebx, dword ptr [ebp+17]

; patched code:

; 0040763F  84C0              test    al, al
; 00407641  E9 A28CFFFF       jmp     004002E8
; 00407646  90                nop
; 00407647  8B5D 11           mov     ebx, dword ptr [ebp+17]

; patched machine code:

jump_patch_2_5	db 0e9h, 0a2h, 08ch, 0ffh, 0ffh	; jmp	04002e8h
		db 090h				; nop
jump_patch_2_5_end:

jump_patch_2_3	db 0e9h, 0d2h, 089h, 0ffh, 0ffh	; jmp	04002e8h
		db 090h				; nop
jump_patch_2_3_end:

%define jump_patch_size jump_patch_2_3_end - jump_patch_2_3	; size is the same as 2_5 code, of course

; part 2 of the patch; the code which part 1 will jump to. this will be placed
; at offset 02e8h in the file, which corresponds to 04002e8h at runtime. this
; is originally filled with 3,352 bytes of zeroes for aligning sections.

%define PATCH_OFFSET 02e8h

patch_2_5:
incbin "patch-2_5.bin"
patch_2_5_end:
%define patch_2_5_size patch_2_5_end - patch_2_5

patch_2_3:
incbin "patch-2_3.bin"
patch_2_3_end:
%define patch_2_3_size patch_2_3_end - patch_2_3

struc version_data
	magic1		resd 2
	magic2_offset	resd 1
	jmppatch_offset	resd 1
	jmppatch	resd 1
	patch		resd 1
	patch_size	resd 1
endstruc

v2_5_data istruc version_data
		at magic1,		dd 040000040h, 0h
		at magic2_offset, 	dd 07630h
		at jmppatch_offset,	dd 07641h
		at jmppatch,		dd jump_patch_2_5
		at patch,		dd patch_2_5
		at patch_size, 		dd patch_2_5_size
iend

v2_3_data istruc version_data
		at magic1, 		dd 0h, 0h
		at magic2_offset,	dd 07900h
		at jmppatch_offset,	dd 07911h
		at jmppatch,		dd jump_patch_2_3
		at patch,		dd patch_2_3
		at patch_size, 		dd patch_2_3_size
iend


; --- patch applier code ---

section .code

; print string %1 defined with "string" macro (or if %1_size is defined manually)
; edi: stdout handle
%macro print 1
		invoke	WriteConsoleA, edi, %1, dword %1_size, dword dummy, byte 0
%endmacro

; print string %1 with length %2
; edi: stdout handle
%macro print 2
		invoke	WriteConsoleA, edi, %1, %2, dword dummy, byte 0
%endmacro

; read a single char from stdin into charbuf
; esi: stdin handle
%define readchar invoke	ReadConsoleA, esi, dword charbuf, byte 1, dword dummy, byte 0

global          _main
_main:
		invoke	GetStdHandle, byte STD_OUTPUT_HANDLE
		mov	edi, eax
		invoke	GetStdHandle, byte STD_INPUT_HANDLE
		mov	esi, eax

		invoke	SetConsoleMode, esi, byte 0
		print	dword greeting
		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY
		print	dword question
		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE

		readchar
		mov	bl, byte [charbuf]

		print	dword charbuf

		cmp	bl, "y"
		je	short .continue
		cmp	bl, "Y"
		je	short .continue
		jmp	.abort
.continue:
		print	dword notice1

		; open the file
		invoke	CreateFileA, dword filename, dword GENERIC_READ | GENERIC_WRITE, byte FILE_SHARE_READ, byte 0, byte OPEN_EXISTING, byte 0, byte 0
		cmp	eax, INVALID_HANDLE_VALUE
		je	.show_last_error
		mov	ebx, eax

		print	dword notice2

		; determine file size
		invoke	GetFileSize, ebx, byte 0
		cmp	eax, -1
		je	.show_last_error

		cmp	eax, GRAVITY_FILE_SIZE_2_5
		je	short .ok5
		cmp	eax, GRAVITY_FILE_SIZE_2_3
		je	short .ok3

		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_INTENSITY
		print	dword error
		print	dword wrong_filesize
		jmp	.quit

.ok5:
		mov	ebp, v2_5_data
		mov	byte [notice3versionminor], '5'
		mov	word [notice3buildx100], '20'
		jmp	short .ok
.ok3:
		mov	ebp, v2_3_data
		mov	byte [notice3versionminor], '3'
		mov	word [notice3buildx100], '18'

.ok:
		print	dword notice3

		; minimalist integrity check

		; check: at offset 02e4h, there should be:
		; 40 00 00 40-00 00 00 00 (2.5)
		; or
		; 00 00 00 00-00 00 00 00 (2.3)

		invoke	SetFilePointer, ebx, dword 02e4h, byte 0, byte FILE_BEGIN
		invoke	ReadFile, ebx, dword buf, byte 8, dword dummy, byte 0
		mov	eax, [ebp+magic1]
		cmp	dword [buf], eax
		jne	.integrity_fail
		mov	eax, [ebp+magic1+4]
		cmp	dword [buf+4], eax
		jne	.integrity_fail


		; check: at offset 07630h (2.5) or 07900h (2.3), there should be:
		; 25 00 00 00-00 83 ec 14-53 55 8b e9-8a 45 0b 84
		; c0 0f 85 e0-02 00 00 8b-5d 11 8b 43-f8 85 c0 0f

		invoke	SetFilePointer, ebx, dword [ebp+magic2_offset], byte 0, byte FILE_BEGIN

		invoke	ReadFile, ebx, dword buf, byte 32, dword dummy, byte 0
		cmp	dword [buf], 000000025h
		jne	.integrity_fail
		cmp	dword [buf+4], 014ec8300h
		jne	.integrity_fail
		cmp	dword [buf+8], 0e98b5553h
		jne	.integrity_fail
		cmp	dword [buf+12], 0840b458ah
		jne	.integrity_fail
		cmp	dword [buf+16], 0e0850fc0h
		jne	.integrity_fail
		cmp	dword [buf+20], 08b000002h
		jne	.integrity_fail
		cmp	dword [buf+24], 0438b115dh
		jne	.integrity_fail
		cmp	dword [buf+28], 00fc085f8h
		jne	.integrity_fail

		print	dword notice4

		; backup

                invoke	CopyFileA, dword filename, dword backup_filename, byte 1
		test	eax, eax
		jz	.backup_fail

		print	dword notice5

		; patch part 1

		invoke	SetFilePointer, ebx, dword [ebp+jmppatch_offset], byte 0, byte FILE_BEGIN
		invoke	WriteFile, ebx, dword [ebp+jmppatch], byte jump_patch_size, dword dummy, byte 0
		test	eax, eax
		jz	.write_fail

		; patch part 2

		invoke	SetFilePointer, ebx, dword PATCH_OFFSET, byte 0, byte FILE_BEGIN
		invoke	WriteFile, ebx, dword [ebp+patch], dword [ebp+patch_size], dword dummy, byte 0
		test	eax, eax
		jz	short .write_fail

		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_GREEN | FOREGROUND_INTENSITY
		print	dword notice6

		invoke	CloseHandle, ebx

		jmp	.no_error

.integrity_fail:
		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_INTENSITY
		print	dword error
		print	dword integrity_fail
		invoke	CloseHandle, ebx
		jmp	.quit

.write_fail:
		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_INTENSITY
		print	dword error_gravity_damaged


.backup_fail:
		invoke	GetLastError
		push	eax
		invoke	CloseHandle, ebx
		pop	eax
		invoke	SetLastError, eax

.show_last_error:
		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_INTENSITY

		invoke	GetLastError
		invoke	FormatMessageA, dword FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_ALLOCATE_BUFFER, byte 0, eax, byte 0, dword err_msg_ptr, byte 0, byte 0
		mov	ebx, eax
		print	dword error
		mov	eax, [err_msg_ptr]
		print	eax, ebx

.quit:
.no_error:
		invoke	SetConsoleTextAttribute, edi, byte FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE
		print	dword done
		readchar

.abort:
		xor	eax, eax
		ret